#
# sp-fishing.R, 10 Feb 24
#
# Data from
# A deep learning model for estimating story points
# Morakot Choetkiertikul and Hoa Khanh Dam and Truyen Tran and Trang Pham and Aditya Ghose∗ and Tim Menzies


library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(4)


# Count and draw line of frequency of story point values
sp_freq=function(df)
{
t_cnt=count(df$storypoint)
t_cnt=subset(t_cnt, freq > 3)

lines(t_cnt, col=pal_col[c_num])
c_num <<- c_num+1
}


# Count and draw line of normalised frequency of story point values
sp_norm=function(df)
{
t_cnt=count(df$storypoint)
t_cnt$norm=1e3*t_cnt$freq/nrow(df) # normalise

t_cnt=subset(t_cnt, freq > 3) # remove 'noise' values

lines(t_cnt$x, t_cnt$norm, col=pal_col[c_num])
c_num <<- c_num+1
}


# Draw line of normalised frequency of story point values
peak_norm=function(df)
{
df$norm=1e3*df$freq/sum(df$freq)
df=subset(df, freq > 3) # remove 'noise' values

lines(df$x, df$norm, col=pal_col[c_num])
c_num <<- c_num+1
}


# Add peak story point value to dataframe
sp_peak=function(df)
{
t_cnt=count(df$storypoint)
# Which frequencies have the maximum value; may be more than one
t_cnt$peak=t_cnt$x[which(t_cnt$freq == max(t_cnt$freq))[1]]

return(t_cnt)
}


# Plot project estimats counts
plot_peak=function(df)
{
plot(0, type="n", log="xy",
	xlim=c(1, 60), ylim=c(1, 1e3),
	xlab="Story points", ylab="Issues (normalised)")
c_num <<- 1

text(10, 1000, paste0("peak=", df$peak[1]))

d_ply(df, .(project), peak_norm)
}


# Percentage of estimates that are not Fibonacci numbers
p_not_fibonacci=function(df)
{
return(100*length(which(!(df$storypoint %in% c(1, 2, 3, 5, 8, 13, 21, 34, 55))))/nrow(df))
}



proj=read.csv("TSE-sp.csv", as.is=TRUE)


proj=read.csv("TSE-sp.csv", as.is=TRUE)
pal_col=rainbow(length(unique(proj$project)))

sort(table(proj$project))
# 
#       jirasoftware             clover           usergrid             bamboo 
#                352                384                482                521 
#          duracloud         mulestudio       aptanastudio          talendesb 
#                666                732                829                868 
#               mule             moodle  talenddataquality              mesos 
#                889               1166               1381               1680 
#           titanium appceleratorstudio           springxd     datamanagement 
#               2251               2919               3526               4667 
mean(table(proj$project))
# [1] 1457.062

plot(0, type="n", log="xy",
	xlim=c(1, 60), ylim=c(1, 100),
	xlab="Story points", ylab="Issues")
c_num=1

# d_ply(proj, .(project), sp_freq)

d_ply(proj, .(project), sp_norm)

p2=ddply(proj, .(project), sp_peak)


par(mfcol=c(2,2))

d_ply(p2, .(peak), plot_peak)

dev.copy(dev=png, file="story-point-peak-cnt.png")
dev.off()


par(mfcol=c(1,1))

not_fib=ddply(proj, .(project), p_not_fibonacci)

plot(sort(not_fib$V1), log="y", col="red",
	yaxs="i",
	ylim=c(0.1, 50),
	xlab="Project", ylab="Estimate not Fibonacci (percent)")

dev.copy(dev=png, file="est-not-fibonacci-num.png")
dev.off()

